# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawProcessorVersionClient, RawProcessorVersionClient
from .types.processor_version_create_request_config import ProcessorVersionCreateRequestConfig
from .types.processor_version_create_request_release_type import ProcessorVersionCreateRequestReleaseType
from .types.processor_version_create_response import ProcessorVersionCreateResponse
from .types.processor_version_get_response import ProcessorVersionGetResponse
from .types.processor_version_list_response import ProcessorVersionListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProcessorVersionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProcessorVersionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProcessorVersionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProcessorVersionClient
        """
        return self._raw_client

    def get(
        self, processor_id: str, processor_version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProcessorVersionGetResponse:
        """
        Retrieve a specific version of a processor in Extend

        Parameters
        ----------
        processor_id : str
            The ID of the processor.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        processor_version_id : str
            The ID of the specific processor version to retrieve.

            Example: `"dpv_QYk6jgHA_8CsO8rVWhyNC"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionGetResponse
            Successfully retrieved processor version

        Examples
        --------
        from extend_ai import Extend
        client = Extend(token="YOUR_TOKEN", )
        client.processor_version.get(processor_id='processor_id_here', processor_version_id='processor_version_id_here', )
        """
        _response = self._raw_client.get(processor_id, processor_version_id, request_options=request_options)
        return _response.data

    def list(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ProcessorVersionListResponse:
        """
        This endpoint allows you to fetch all versions of a given processor, including the current `draft` version.

        Versions are typically returned in descending order of creation (newest first), but this should be confirmed in the actual implementation.
        The `draft` version is the latest unpublished version of the processor, which can be published to create a new version. It might not have any changes from the last published version.

        Parameters
        ----------
        id : str
            The ID of the processor to retrieve versions for.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionListResponse
            Successfully retrieved processor versions

        Examples
        --------
        from extend_ai import Extend
        client = Extend(token="YOUR_TOKEN", )
        client.processor_version.list(id='processor_id_here', )
        """
        _response = self._raw_client.list(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        release_type: ProcessorVersionCreateRequestReleaseType,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[ProcessorVersionCreateRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorVersionCreateResponse:
        """
        This endpoint allows you to publish a new version of an existing processor. Publishing a new version creates a snapshot of the processor's current configuration and makes it available for use in workflows.

        Publishing a new version does not automatically update existing workflows using this processor. You may need to manually update workflows to use the new version if desired.

        Parameters
        ----------
        id : str
            The ID of the processor to publish a new version for.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        release_type : ProcessorVersionCreateRequestReleaseType
            The type of release for this version. The two options are "major" and "minor", which will increment the version number accordingly.

        description : typing.Optional[str]
            A description of the changes in this version. This helps track the evolution of the processor over time.

        config : typing.Optional[ProcessorVersionCreateRequestConfig]
            The configuration for this version of the processor. The type of configuration must match the processor type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionCreateResponse
            Successfully published processor version

        Examples
        --------
        from extend_ai import Extend
        client = Extend(token="YOUR_TOKEN", )
        client.processor_version.create(id='processor_id_here', release_type="major", )
        """
        _response = self._raw_client.create(
            id, release_type=release_type, description=description, config=config, request_options=request_options
        )
        return _response.data


class AsyncProcessorVersionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProcessorVersionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProcessorVersionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProcessorVersionClient
        """
        return self._raw_client

    async def get(
        self, processor_id: str, processor_version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProcessorVersionGetResponse:
        """
        Retrieve a specific version of a processor in Extend

        Parameters
        ----------
        processor_id : str
            The ID of the processor.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        processor_version_id : str
            The ID of the specific processor version to retrieve.

            Example: `"dpv_QYk6jgHA_8CsO8rVWhyNC"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionGetResponse
            Successfully retrieved processor version

        Examples
        --------
        from extend_ai import AsyncExtend
        import asyncio
        client = AsyncExtend(token="YOUR_TOKEN", )
        async def main() -> None:
            await client.processor_version.get(processor_id='processor_id_here', processor_version_id='processor_version_id_here', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get(processor_id, processor_version_id, request_options=request_options)
        return _response.data

    async def list(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ProcessorVersionListResponse:
        """
        This endpoint allows you to fetch all versions of a given processor, including the current `draft` version.

        Versions are typically returned in descending order of creation (newest first), but this should be confirmed in the actual implementation.
        The `draft` version is the latest unpublished version of the processor, which can be published to create a new version. It might not have any changes from the last published version.

        Parameters
        ----------
        id : str
            The ID of the processor to retrieve versions for.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionListResponse
            Successfully retrieved processor versions

        Examples
        --------
        from extend_ai import AsyncExtend
        import asyncio
        client = AsyncExtend(token="YOUR_TOKEN", )
        async def main() -> None:
            await client.processor_version.list(id='processor_id_here', )
        asyncio.run(main())
        """
        _response = await self._raw_client.list(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        release_type: ProcessorVersionCreateRequestReleaseType,
        description: typing.Optional[str] = OMIT,
        config: typing.Optional[ProcessorVersionCreateRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProcessorVersionCreateResponse:
        """
        This endpoint allows you to publish a new version of an existing processor. Publishing a new version creates a snapshot of the processor's current configuration and makes it available for use in workflows.

        Publishing a new version does not automatically update existing workflows using this processor. You may need to manually update workflows to use the new version if desired.

        Parameters
        ----------
        id : str
            The ID of the processor to publish a new version for.

            Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`

        release_type : ProcessorVersionCreateRequestReleaseType
            The type of release for this version. The two options are "major" and "minor", which will increment the version number accordingly.

        description : typing.Optional[str]
            A description of the changes in this version. This helps track the evolution of the processor over time.

        config : typing.Optional[ProcessorVersionCreateRequestConfig]
            The configuration for this version of the processor. The type of configuration must match the processor type.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProcessorVersionCreateResponse
            Successfully published processor version

        Examples
        --------
        from extend_ai import AsyncExtend
        import asyncio
        client = AsyncExtend(token="YOUR_TOKEN", )
        async def main() -> None:
            await client.processor_version.create(id='processor_id_here', release_type="major", )
        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            id, release_type=release_type, description=description, config=config, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .processor_type import ProcessorType
from .processor_version_config import ProcessorVersionConfig


class ProcessorVersion(UncheckedBaseModel):
    object: str = pydantic.Field()
    """
    The type of the object. In this case, it will always be `"document_processor_version"`.
    """

    id: str = pydantic.Field()
    """
    The unique identifier for this version of the document processor.
    
    Example: `"dpv_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    processor_id: typing_extensions.Annotated[str, FieldMetadata(alias="processorId")] = pydantic.Field()
    """
    The ID of the parent document processor.
    
    Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    processor_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="processorName")] = (
        pydantic.Field(default=None)
    )
    """
    The name of the parent document processor.
    
    Example: `"Invoice Processor"`
    """

    processor_type: typing_extensions.Annotated[ProcessorType, FieldMetadata(alias="processorType")]
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional description of this version of the document processor.
    
    Example: `"Updated extraction fields for new invoice format"`
    """

    version: str = pydantic.Field()
    """
    The version number or identifier for this specific version of the document processor. The draft version will have version="draft".
    
    Examples: `"1.0"`, `"2.1"`, `"draft"`
    """

    config: ProcessorVersionConfig = pydantic.Field()
    """
    The configuration settings for this version of the document processor. The structure of this object will vary depending on the processor type.
    
    See the configuration section in the "Guides" tab, for yout specific processor type, for more details on the configuration settings.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The time (in UTC) at which this version of the document processor was created. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    The time (in UTC) at which this version of the document processor was last updated. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .extraction_field import ExtractionField  # noqa: E402, F401, I001

update_forward_refs(ProcessorVersion)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from ...types.provided_processor_output import ProvidedProcessorOutput


class EvaluationSetItemCreateBatchRequestItemsItem(UncheckedBaseModel):
    file_id: typing_extensions.Annotated[str, FieldMetadata(alias="fileId")] = pydantic.Field()
    """
    Extend's internal ID for the file. It will always start with "file_".
    
    Example: `"file_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    expected_output: typing_extensions.Annotated[ProvidedProcessorOutput, FieldMetadata(alias="expectedOutput")] = (
        pydantic.Field()
    )
    """
    The expected output of the processor when run against the file
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .processor_type import ProcessorType
from .processor_version import ProcessorVersion


class Processor(UncheckedBaseModel):
    object: str = pydantic.Field()
    """
    The type of response. In this case, it will always be `"document_processor"`.
    """

    id: str = pydantic.Field()
    """
    The ID of the processor.
    
    Example: `"dp_Xj8mK2pL9nR4vT7qY5wZ"`
    """

    name: str = pydantic.Field()
    """
    The name of the processor.
    
    Example: `"Invoice Processor"`
    """

    type: ProcessorType
    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The time (in UTC) at which the processor was created. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    The time (in UTC) at which the processor was last updated. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    draft_version: typing_extensions.Annotated[ProcessorVersion, FieldMetadata(alias="draftVersion")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .extraction_field import ExtractionField  # noqa: E402, F401, I001

update_forward_refs(Processor)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .json_output_metadata_value_citations_item import JsonOutputMetadataValueCitationsItem
from .json_output_metadata_value_insights_item import JsonOutputMetadataValueInsightsItem


class JsonOutputMetadataValue(UncheckedBaseModel):
    ocr_confidence: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="ocrConfidence")] = (
        pydantic.Field(default=None)
    )
    """
    Confidence score from OCR processing, if applicable
    """

    logprobs_confidence: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="logprobsConfidence")
    ] = pydantic.Field(default=None)
    """
    Confidence score based on model logprobs
    """

    citations: typing.Optional[typing.List[JsonOutputMetadataValueCitationsItem]] = None
    insights: typing.Optional[typing.List[JsonOutputMetadataValueInsightsItem]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

StepRunOutputRulesItemFailureReason = typing.Union[
    typing.Literal["RULE_FAILED", "PARSE_ERROR", "VALUE_ERROR"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawBatchWorkflowRunClient, RawBatchWorkflowRunClient
from .types.batch_workflow_run_create_request_inputs_item import BatchWorkflowRunCreateRequestInputsItem
from .types.batch_workflow_run_create_response import BatchWorkflowRunCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BatchWorkflowRunClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBatchWorkflowRunClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBatchWorkflowRunClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBatchWorkflowRunClient
        """
        return self._raw_client

    def create(
        self,
        *,
        workflow_id: str,
        inputs: typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchWorkflowRunCreateResponse:
        """
        This endpoint allows you to efficiently initiate large batches of workflow runs in a single request (up to 1,000 in a single request, but you can queue up multiple batches in rapid succession). It accepts an array of inputs, each containing a file and metadata pair. The primary use case for this endpoint is for doing large bulk runs of >1000 files at a time that can process over the course of a few hours without needing to manage rate limits that would likely occur using the primary run endpoint.

        Unlike the single [Run Workflow](/developers/api-reference/workflow-endpoints/run-workflow) endpoint which returns the details of the created workflow runs immediately, this batch endpoint returns a `batchId`.

        Our recommended usage pattern is to integrate with [Webhooks](/product/webhooks/configuration) for consuming results, using the `metadata` and `batchId` to match up results to the original inputs in your downstream systems. However, you can integrate in a polling mechanism by using a combination of the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint to fetch all runs via a batch, and then [Get Workflow Run](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/get-workflow-run) to fetch the full outputs each run.

        **Priority:** All workflow runs created through this batch endpoint are automatically assigned a priority of 90.

        **Processing and Monitoring:**
        Upon successful submission, the endpoint returns a `batchId`. The individual workflow runs are then queued for processing.

        - **Monitoring:** Track the progress and consume results of individual runs using [Webhooks](/product/webhooks/configuration). Subscribe to events like `workflow_run.completed`, `workflow_run.failed`, etc. The webhook payload for these events will include the corresponding `batchId` and the `metadata` you provided for each input.
        - **Fetching Results:** You can also use the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint and filter using the `batchId` query param.

        Parameters
        ----------
        workflow_id : str
            The ID of the workflow to run. This ID will start with "workflow_". This ID can be found viewing the workflow on the Extend platform.

            Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`

        inputs : typing.Sequence[BatchWorkflowRunCreateRequestInputsItem]
            An array of input objects to be processed by the workflow. Each object represents a single workflow run to be created. The array must contain at least 1 input and at most 1000 inputs.

        version : typing.Optional[str]
            An optional version of the workflow to use. This can be a specific version number (e.g., `"1"`, `"2"`) found on the Extend platform, or `"draft"` to use the current unpublished draft version. When a version is not supplied, the latest deployed version of the workflow will be used. If no deployed version exists, the draft version will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchWorkflowRunCreateResponse
            Successfully queued batch workflow run

        Examples
        --------
        from extend_ai import Extend
        from extend_ai.batch_workflow_run import BatchWorkflowRunCreateRequestInputsItem
        client = Extend(token="YOUR_TOKEN", )
        client.batch_workflow_run.create(workflow_id='workflow_id_here', inputs=[BatchWorkflowRunCreateRequestInputsItem()], )
        """
        _response = self._raw_client.create(
            workflow_id=workflow_id, inputs=inputs, version=version, request_options=request_options
        )
        return _response.data


class AsyncBatchWorkflowRunClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBatchWorkflowRunClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBatchWorkflowRunClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBatchWorkflowRunClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        workflow_id: str,
        inputs: typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchWorkflowRunCreateResponse:
        """
        This endpoint allows you to efficiently initiate large batches of workflow runs in a single request (up to 1,000 in a single request, but you can queue up multiple batches in rapid succession). It accepts an array of inputs, each containing a file and metadata pair. The primary use case for this endpoint is for doing large bulk runs of >1000 files at a time that can process over the course of a few hours without needing to manage rate limits that would likely occur using the primary run endpoint.

        Unlike the single [Run Workflow](/developers/api-reference/workflow-endpoints/run-workflow) endpoint which returns the details of the created workflow runs immediately, this batch endpoint returns a `batchId`.

        Our recommended usage pattern is to integrate with [Webhooks](/product/webhooks/configuration) for consuming results, using the `metadata` and `batchId` to match up results to the original inputs in your downstream systems. However, you can integrate in a polling mechanism by using a combination of the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint to fetch all runs via a batch, and then [Get Workflow Run](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/get-workflow-run) to fetch the full outputs each run.

        **Priority:** All workflow runs created through this batch endpoint are automatically assigned a priority of 90.

        **Processing and Monitoring:**
        Upon successful submission, the endpoint returns a `batchId`. The individual workflow runs are then queued for processing.

        - **Monitoring:** Track the progress and consume results of individual runs using [Webhooks](/product/webhooks/configuration). Subscribe to events like `workflow_run.completed`, `workflow_run.failed`, etc. The webhook payload for these events will include the corresponding `batchId` and the `metadata` you provided for each input.
        - **Fetching Results:** You can also use the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint and filter using the `batchId` query param.

        Parameters
        ----------
        workflow_id : str
            The ID of the workflow to run. This ID will start with "workflow_". This ID can be found viewing the workflow on the Extend platform.

            Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`

        inputs : typing.Sequence[BatchWorkflowRunCreateRequestInputsItem]
            An array of input objects to be processed by the workflow. Each object represents a single workflow run to be created. The array must contain at least 1 input and at most 1000 inputs.

        version : typing.Optional[str]
            An optional version of the workflow to use. This can be a specific version number (e.g., `"1"`, `"2"`) found on the Extend platform, or `"draft"` to use the current unpublished draft version. When a version is not supplied, the latest deployed version of the workflow will be used. If no deployed version exists, the draft version will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchWorkflowRunCreateResponse
            Successfully queued batch workflow run

        Examples
        --------
        from extend_ai import AsyncExtend
        from extend_ai.batch_workflow_run import BatchWorkflowRunCreateRequestInputsItem
        import asyncio
        client = AsyncExtend(token="YOUR_TOKEN", )
        async def main() -> None:
            await client.batch_workflow_run.create(workflow_id='workflow_id_here', inputs=[BatchWorkflowRunCreateRequestInputsItem()], )
        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            workflow_id=workflow_id, inputs=inputs, version=version, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel


class TableCellDetails(UncheckedBaseModel):
    """
    Details for table cell blocks
    """

    type: typing.Literal["table_cell_details"] = pydantic.Field(default="table_cell_details")
    """
    Indicates this is a table cell details object
    """

    row_index: typing_extensions.Annotated[int, FieldMetadata(alias="rowIndex")]
    column_index: typing_extensions.Annotated[int, FieldMetadata(alias="columnIndex")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from ...types.classification import Classification
from ...types.classification_advanced_options import ClassificationAdvancedOptions
from ...types.classification_config_base_processor import ClassificationConfigBaseProcessor
from ...types.extraction_advanced_options import ExtractionAdvancedOptions
from ...types.extraction_config_base_processor import ExtractionConfigBaseProcessor
from ...types.json_object import JsonObject
from ...types.parse_config import ParseConfig
from ...types.splitter_advanced_options import SplitterAdvancedOptions
from ...types.splitter_config_base_processor import SplitterConfigBaseProcessor


class ProcessorRunCreateRequestConfig_Classify(UncheckedBaseModel):
    """
    The configuration for the processor run. If this is provided, this config will be used. If not provided, the config for the specific version you provide will be used. The type of configuration must match the processor type.
    """

    type: typing.Literal["CLASSIFY"] = "CLASSIFY"
    base_processor: typing_extensions.Annotated[
        typing.Optional[ClassificationConfigBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = None
    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = None
    classifications: typing.List[Classification]
    classification_rules: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="classificationRules")
    ] = None
    advanced_options: typing_extensions.Annotated[
        typing.Optional[ClassificationAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = None
    parser: typing.Optional[ParseConfig] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ProcessorRunCreateRequestConfig_Extract(UncheckedBaseModel):
    """
    The configuration for the processor run. If this is provided, this config will be used. If not provided, the config for the specific version you provide will be used. The type of configuration must match the processor type.
    """

    type: typing.Literal["EXTRACT"] = "EXTRACT"
    base_processor: typing_extensions.Annotated[
        typing.Optional[ExtractionConfigBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = None
    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = None
    extraction_rules: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="extractionRules")] = None
    schema_: typing_extensions.Annotated[typing.Optional[JsonObject], FieldMetadata(alias="schema")] = None
    fields: typing.Optional[typing.List["ExtractionField"]] = None
    advanced_options: typing_extensions.Annotated[
        typing.Optional[ExtractionAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = None
    parser: typing.Optional[ParseConfig] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...types.extraction_field import ExtractionField  # noqa: E402, F401, I001


class ProcessorRunCreateRequestConfig_Splitter(UncheckedBaseModel):
    """
    The configuration for the processor run. If this is provided, this config will be used. If not provided, the config for the specific version you provide will be used. The type of configuration must match the processor type.
    """

    type: typing.Literal["SPLITTER"] = "SPLITTER"
    base_processor: typing_extensions.Annotated[
        typing.Optional[SplitterConfigBaseProcessor], FieldMetadata(alias="baseProcessor")
    ] = None
    base_version: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="baseVersion")] = None
    split_classifications: typing_extensions.Annotated[
        typing.List[Classification], FieldMetadata(alias="splitClassifications")
    ]
    split_rules: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="splitRules")] = None
    advanced_options: typing_extensions.Annotated[
        typing.Optional[SplitterAdvancedOptions], FieldMetadata(alias="advancedOptions")
    ] = None
    parser: typing.Optional[ParseConfig] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ProcessorRunCreateRequestConfig = typing_extensions.Annotated[
    typing.Union[
        ProcessorRunCreateRequestConfig_Classify,
        ProcessorRunCreateRequestConfig_Extract,
        ProcessorRunCreateRequestConfig_Splitter,
    ],
    UnionMetadata(discriminant="type"),
]
update_forward_refs(ProcessorRunCreateRequestConfig_Extract)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.error import Error
from .types.workflow_create_response import WorkflowCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWorkflowClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[WorkflowCreateResponse]:
        """
        Create a new workflow in Extend. Workflows are sequences of steps that process files and data in a specific order to achieve a desired outcome.

        This endpoint will create a new workflow in Extend, which can then be configured and deployed. Typically, workflows are created from our UI, however this endpoint can be used to create workflows programmatically. Configuration of the flow still needs to be done in the dashboard.

        Parameters
        ----------
        name : str
            The name of the workflow

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WorkflowCreateResponse]
            Successfully created workflow
        """
        _response = self._client_wrapper.httpx_client.request(
            "workflows",
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkflowCreateResponse,
                    construct_type(
                        type_=WorkflowCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        construct_type(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawWorkflowClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[WorkflowCreateResponse]:
        """
        Create a new workflow in Extend. Workflows are sequences of steps that process files and data in a specific order to achieve a desired outcome.

        This endpoint will create a new workflow in Extend, which can then be configured and deployed. Typically, workflows are created from our UI, however this endpoint can be used to create workflows programmatically. Configuration of the flow still needs to be done in the dashboard.

        Parameters
        ----------
        name : str
            The name of the workflow

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WorkflowCreateResponse]
            Successfully created workflow
        """
        _response = await self._client_wrapper.httpx_client.request(
            "workflows",
            method="POST",
            json={
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WorkflowCreateResponse,
                    construct_type(
                        type_=WorkflowCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        construct_type(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .webhook_event_event_type import WebhookEventEventType
from .webhook_event_payload import WebhookEventPayload


class WebhookEvent(UncheckedBaseModel):
    event_id: typing_extensions.Annotated[str, FieldMetadata(alias="eventId")] = pydantic.Field()
    """
    Unique identifier for the event
    """

    event_type: typing_extensions.Annotated[WebhookEventEventType, FieldMetadata(alias="eventType")] = pydantic.Field()
    """
    Type of the event that occurred
    """

    payload: WebhookEventPayload = pydantic.Field()
    """
    Contains the relevant object for the event type
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .extraction_field import ExtractionField  # noqa: E402, F401, I001

update_forward_refs(WebhookEvent)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .chunk import Chunk
from .parse_config import ParseConfig
from .parser_run_metrics import ParserRunMetrics
from .parser_run_status_enum import ParserRunStatusEnum


class ParserRun(UncheckedBaseModel):
    """
    Full parser run object with complete parsing results
    """

    id: str = pydantic.Field()
    """
    A unique identifier for the parser run. Will always start with `"parser_run_"`
    
    Example: `"parser_run_xK9mLPqRtN3vS8wF5hB2cQ"`
    """

    file_id: typing_extensions.Annotated[str, FieldMetadata(alias="fileId")] = pydantic.Field()
    """
    The identifier of the file that was parsed. This can be used as a parameter to other Extend endpoints, such as processor runs.
    """

    chunks: typing.List[Chunk] = pydantic.Field()
    """
    An array of chunks that were parsed from the file.
    """

    status: ParserRunStatusEnum = pydantic.Field()
    """
    The status of the parser run:
    * `"PROCESSED"` - The file was successfully processed
    * `"FAILED"` - The processing failed (see failureReason for details)
    """

    failure_reason: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="failureReason")] = (
        pydantic.Field(default=None)
    )
    """
    The reason for failure if status is "FAILED".
    """

    metrics: ParserRunMetrics = pydantic.Field()
    """
    Metrics about the parsing process.
    """

    config: ParseConfig = pydantic.Field()
    """
    The configuration used for the parsing process, including any default values that were applied.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .provided_fields_array_output import ProvidedFieldsArrayOutput
from .provided_json_output import ProvidedJsonOutput

ProvidedExtractionOutput = typing.Union[ProvidedJsonOutput, ProvidedFieldsArrayOutput]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .extract_chunking_options import ExtractChunkingOptions
from .page_ranges import PageRanges


class ExtractionAdvancedOptions(UncheckedBaseModel):
    document_kind: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="documentKind")] = (
        pydantic.Field(default=None)
    )
    """
    The kind of document being processed.
    """

    key_definitions: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="keyDefinitions")] = (
        pydantic.Field(default=None)
    )
    """
    Custom key definitions for extraction.
    """

    model_reasoning_insights_enabled: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="modelReasoningInsightsEnabled")
    ] = pydantic.Field(default=None)
    """
    Whether to enable model reasoning insights.
    """

    advanced_multimodal_enabled: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="advancedMultimodalEnabled")
    ] = pydantic.Field(default=None)
    """
    Whether to enable advanced multimodal features.
    """

    citations_enabled: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="citationsEnabled")] = (
        pydantic.Field(default=None)
    )
    """
    Whether to enable citations in the output.
    """

    advanced_figure_parsing_enabled: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="advancedFigureParsingEnabled")
    ] = pydantic.Field(default=None)
    """
    Whether to enable advanced figure parsing.
    """

    chunking_options: typing_extensions.Annotated[
        typing.Optional[ExtractChunkingOptions], FieldMetadata(alias="chunkingOptions")
    ] = None
    fixed_page_limit: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="fixedPageLimit")] = (
        pydantic.Field(default=None)
    )
    """
    Optional fixed limit on the number of pages to process. See [Page Ranges](/product/page-ranges).
    """

    page_ranges: typing_extensions.Annotated[typing.Optional[PageRanges], FieldMetadata(alias="pageRanges")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .workflow_status import WorkflowStatus


class WorkflowRunSummary(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the workflow run.
    
    Example: `"workflow_run_Zk9mNP12Qw4-yTv8BdR3H"`
    """

    status: WorkflowStatus
    initial_run_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="initialRunAt")] = (
        pydantic.Field(default=None)
    )
    """
    The time (in UTC) at which the workflow was initially created. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    reviewed_by_user: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="reviewedByUser")] = (
        pydantic.Field(default=None)
    )
    """
    The user of the person who reviewed the workflow run. Will not be included if the workflow run has not been reviewed.
    
    Example: `"jane.doe@example.com"`
    """

    reviewed_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="reviewedAt")] = (
        pydantic.Field(default=None)
    )
    """
    The time (in UTC) at which the workflow run was reviewed. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    start_time: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="startTime")] = (
        pydantic.Field(default=None)
    )
    """
    The start time (in UTC) that the workflow actually started executing. This occurs after the `initialRunAt` time. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:30:00Z"`
    """

    end_time: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="endTime")] = (
        pydantic.Field(default=None)
    )
    """
    The end time (in UTC) that the workflow finished. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:35:00Z"`
    """

    workflow_id: typing_extensions.Annotated[str, FieldMetadata(alias="workflowId")] = pydantic.Field()
    """
    The ID of the workflow that was run. Will always start with "workflow".
    
    Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`
    """

    workflow_name: typing_extensions.Annotated[str, FieldMetadata(alias="workflowName")] = pydantic.Field()
    """
    The name of the workflow that was run.
    
    Example: `"Invoice Processing"`
    """

    workflow_version_id: typing_extensions.Annotated[str, FieldMetadata(alias="workflowVersionId")] = pydantic.Field()
    """
    The ID of the workflow version that was run. Will always start with "workflow_version".
    
    Example: `"workflow_version_Zk9mNP12Qw4-yTv8BdR3H"`
    """

    batch_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="batchId")] = pydantic.Field(
        default=None
    )
    """
    The batch ID of the WorkflowRun. If that WorkflowRun was created from a batch of files, all runs in that batch will have the same batch ID.
    
    Example: `"batch_7Ws31-F5"`
    """

    rejection_note: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="rejectionNote")] = (
        pydantic.Field(default=None)
    )
    """
    The note that was added when the workflow run was rejected.
    
    Example: `"Invalid invoice format"`
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The time (in UTC) at which the workflow run was created. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T15:29:55Z"`
    """

    updated_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="updatedAt")] = pydantic.Field()
    """
    The time (in UTC) at which the workflow run was last updated. Will follow the RFC 3339 format.
    
    Example: `"2024-03-21T16:45:00Z"`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from .http_client import AsyncHttpClient, HttpClient


class BaseClientWrapper:
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
        timeout: typing.Optional[float] = None,
        extend_api_version: typing.Optional[str] = None,
    ):
        self._token = token
        self._base_url = base_url
        self._timeout = timeout
        self._extend_api_version = extend_api_version

    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "User-Agent": "extend_ai/0.0.7",
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "extend_ai",
            "X-Fern-SDK-Version": "0.0.7",
        }
        headers["Authorization"] = f"Bearer {self._get_token()}"
        headers["x-extend-api-version"] = (
            self._extend_api_version if self._extend_api_version is not None else "2025-04-21"
        )
        return headers

    def _get_token(self) -> str:
        if isinstance(self._token, str):
            return self._token
        else:
            return self._token()

    def get_base_url(self) -> str:
        return self._base_url

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
        timeout: typing.Optional[float] = None,
        extend_api_version: typing.Optional[str] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(token=token, base_url=base_url, timeout=timeout, extend_api_version=extend_api_version)
        self.httpx_client = HttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        base_url: str,
        timeout: typing.Optional[float] = None,
        extend_api_version: typing.Optional[str] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(token=token, base_url=base_url, timeout=timeout, extend_api_version=extend_api_version)
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.error import Error
from .types.batch_workflow_run_create_request_inputs_item import BatchWorkflowRunCreateRequestInputsItem
from .types.batch_workflow_run_create_response import BatchWorkflowRunCreateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBatchWorkflowRunClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        workflow_id: str,
        inputs: typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchWorkflowRunCreateResponse]:
        """
        This endpoint allows you to efficiently initiate large batches of workflow runs in a single request (up to 1,000 in a single request, but you can queue up multiple batches in rapid succession). It accepts an array of inputs, each containing a file and metadata pair. The primary use case for this endpoint is for doing large bulk runs of >1000 files at a time that can process over the course of a few hours without needing to manage rate limits that would likely occur using the primary run endpoint.

        Unlike the single [Run Workflow](/developers/api-reference/workflow-endpoints/run-workflow) endpoint which returns the details of the created workflow runs immediately, this batch endpoint returns a `batchId`.

        Our recommended usage pattern is to integrate with [Webhooks](/product/webhooks/configuration) for consuming results, using the `metadata` and `batchId` to match up results to the original inputs in your downstream systems. However, you can integrate in a polling mechanism by using a combination of the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint to fetch all runs via a batch, and then [Get Workflow Run](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/get-workflow-run) to fetch the full outputs each run.

        **Priority:** All workflow runs created through this batch endpoint are automatically assigned a priority of 90.

        **Processing and Monitoring:**
        Upon successful submission, the endpoint returns a `batchId`. The individual workflow runs are then queued for processing.

        - **Monitoring:** Track the progress and consume results of individual runs using [Webhooks](/product/webhooks/configuration). Subscribe to events like `workflow_run.completed`, `workflow_run.failed`, etc. The webhook payload for these events will include the corresponding `batchId` and the `metadata` you provided for each input.
        - **Fetching Results:** You can also use the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint and filter using the `batchId` query param.

        Parameters
        ----------
        workflow_id : str
            The ID of the workflow to run. This ID will start with "workflow_". This ID can be found viewing the workflow on the Extend platform.

            Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`

        inputs : typing.Sequence[BatchWorkflowRunCreateRequestInputsItem]
            An array of input objects to be processed by the workflow. Each object represents a single workflow run to be created. The array must contain at least 1 input and at most 1000 inputs.

        version : typing.Optional[str]
            An optional version of the workflow to use. This can be a specific version number (e.g., `"1"`, `"2"`) found on the Extend platform, or `"draft"` to use the current unpublished draft version. When a version is not supplied, the latest deployed version of the workflow will be used. If no deployed version exists, the draft version will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchWorkflowRunCreateResponse]
            Successfully queued batch workflow run
        """
        _response = self._client_wrapper.httpx_client.request(
            "workflow_runs/batch",
            method="POST",
            json={
                "workflowId": workflow_id,
                "version": version,
                "inputs": convert_and_respect_annotation_metadata(
                    object_=inputs,
                    annotation=typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchWorkflowRunCreateResponse,
                    construct_type(
                        type_=BatchWorkflowRunCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        construct_type(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBatchWorkflowRunClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        workflow_id: str,
        inputs: typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
        version: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchWorkflowRunCreateResponse]:
        """
        This endpoint allows you to efficiently initiate large batches of workflow runs in a single request (up to 1,000 in a single request, but you can queue up multiple batches in rapid succession). It accepts an array of inputs, each containing a file and metadata pair. The primary use case for this endpoint is for doing large bulk runs of >1000 files at a time that can process over the course of a few hours without needing to manage rate limits that would likely occur using the primary run endpoint.

        Unlike the single [Run Workflow](/developers/api-reference/workflow-endpoints/run-workflow) endpoint which returns the details of the created workflow runs immediately, this batch endpoint returns a `batchId`.

        Our recommended usage pattern is to integrate with [Webhooks](/product/webhooks/configuration) for consuming results, using the `metadata` and `batchId` to match up results to the original inputs in your downstream systems. However, you can integrate in a polling mechanism by using a combination of the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint to fetch all runs via a batch, and then [Get Workflow Run](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/get-workflow-run) to fetch the full outputs each run.

        **Priority:** All workflow runs created through this batch endpoint are automatically assigned a priority of 90.

        **Processing and Monitoring:**
        Upon successful submission, the endpoint returns a `batchId`. The individual workflow runs are then queued for processing.

        - **Monitoring:** Track the progress and consume results of individual runs using [Webhooks](/product/webhooks/configuration). Subscribe to events like `workflow_run.completed`, `workflow_run.failed`, etc. The webhook payload for these events will include the corresponding `batchId` and the `metadata` you provided for each input.
        - **Fetching Results:** You can also use the [List Workflow Runs](https://docs.extend.ai/2025-04-21/developers/api-reference/workflow-endpoints/list-workflow-runs) endpoint and filter using the `batchId` query param.

        Parameters
        ----------
        workflow_id : str
            The ID of the workflow to run. This ID will start with "workflow_". This ID can be found viewing the workflow on the Extend platform.

            Example: `"workflow_BMdfq_yWM3sT-ZzvCnA3f"`

        inputs : typing.Sequence[BatchWorkflowRunCreateRequestInputsItem]
            An array of input objects to be processed by the workflow. Each object represents a single workflow run to be created. The array must contain at least 1 input and at most 1000 inputs.

        version : typing.Optional[str]
            An optional version of the workflow to use. This can be a specific version number (e.g., `"1"`, `"2"`) found on the Extend platform, or `"draft"` to use the current unpublished draft version. When a version is not supplied, the latest deployed version of the workflow will be used. If no deployed version exists, the draft version will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchWorkflowRunCreateResponse]
            Successfully queued batch workflow run
        """
        _response = await self._client_wrapper.httpx_client.request(
            "workflow_runs/batch",
            method="POST",
            json={
                "workflowId": workflow_id,
                "version": version,
                "inputs": convert_and_respect_annotation_metadata(
                    object_=inputs,
                    annotation=typing.Sequence[BatchWorkflowRunCreateRequestInputsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchWorkflowRunCreateResponse,
                    construct_type(
                        type_=BatchWorkflowRunCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        construct_type(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        construct_type(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .step_run_output_rules_item_failure_reason import StepRunOutputRulesItemFailureReason


class StepRunOutputRulesItem(UncheckedBaseModel):
    name: str = pydantic.Field()
    """
    The name of the validation rule.
    """

    valid: bool = pydantic.Field()
    """
    Indicates whether this validation rule passed or not. This field will be `true` only if the formula evaluates to `true`. If the rule's formula is array valued, then this field will only be `true` if the formula evaluates to `true` for every item in the array.
    """

    valid_array: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="validArray")] = (
        pydantic.Field(default=None)
    )
    """
    Only present if the validation rule's formula is array valued. This field contains the formula's evaluated result for every item in the array.
    """

    failure_reason: typing_extensions.Annotated[
        typing.Optional[StepRunOutputRulesItemFailureReason], FieldMetadata(alias="failureReason")
    ] = pydantic.Field(default=None)
    """
    If the validation rule is not valid, then this describes why the rule failed.
    * `"RULE_FAILED"` - The formula evaluated to `false` or `null`
    * `"PARSE_ERROR"` - The formula could not be parsed
    * `"VALUE_ERROR"` - An error occurred while evaluating the formula
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the `failureReason` is `PARSE_ERROR` or `VALUE_ERROR`, then this field contains the error's details.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BadRequestErrorBodyCode = typing.Union[
    typing.Literal["FILE_TYPE_NOT_SUPPORTED", "FILE_SIZE_TOO_LARGE", "CORRUPT_FILE", "PASSWORD_PROTECTED_FILE"],
    typing.Any,
]
